@@ -35,6 +35,7 @@
 #include <gnutls_datum.h>
 #include <extras/randomart.h>
 #include <read-file.h>
+#include <algorithms.h>
 
 /**
  * gnutls_random_art:
@@ -94,7 +95,7 @@ int gnutls_random_art (gnutls_random_art_t type,
  * Note that since 3.1.7 this function is deprecated. The minimum
  * number of bits is set by the priority string level.
  * Also this function must be called after gnutls_priority_set_direct()
- * or the set value may be overriden by the selected priority options.
+ * or the set value may be overridden by the selected priority options.
  *
  *
  **/
@@ -584,7 +585,7 @@ gnutls_fingerprint (gnutls_digest_algorithm_t algo,
                     size_t * result_size)
 {
   int ret;
-  int hash_len = _gnutls_hash_get_algo_len (algo);
+  int hash_len = _gnutls_hash_get_algo_len (mac_to_entry(algo));
 
   if (hash_len < 0 || (unsigned) hash_len > *result_size || result == NULL)
     {
@@ -849,12 +850,12 @@ gnutls_session_get_desc (gnutls_session_t session)
 
     type = gnutls_certificate_type_get (session);
     if (type == GNUTLS_CRT_X509)
-      snprintf(proto_name, sizeof(proto_name), "%s-PKIX", gnutls_protocol_get_name(_gnutls_protocol_get_version(session)));
+      snprintf(proto_name, sizeof(proto_name), "%s-PKIX", gnutls_protocol_get_name(get_num_version(session)));
     else
-      snprintf(proto_name, sizeof(proto_name), "%s-%s", gnutls_protocol_get_name(_gnutls_protocol_get_version(session)),
+      snprintf(proto_name, sizeof(proto_name), "%s-%s", gnutls_protocol_get_name(get_num_version(session)),
                                                         gnutls_certificate_type_get_name(type));
     
-    gnutls_protocol_get_name(_gnutls_protocol_get_version (session)),
+    gnutls_protocol_get_name(get_num_version (session)),
 
     desc = gnutls_malloc(DESC_SIZE);
     if (desc == NULL)
@@ -869,3 +870,35 @@ gnutls_session_get_desc (gnutls_session_t session)
             
     return desc;
 }
+
+/**
+ * gnutls_session_set_id:
+ * @session: is a #gnutls_session_t structure.
+ * @sid: the session identifier
+ *
+ * This function sets the session ID to be used in a client hello.
+ * This is a function intended for exceptional uses. Do not use this
+ * function unless you are implementing a custom protocol.
+ *
+ * To set session resumption parameters use gnutls_session_set_data() instead.
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
+ *   an error code is returned.
+ **/
+int
+gnutls_session_set_id (gnutls_session_t session, 
+                              const gnutls_datum_t * sid)
+{
+  if (session->security_parameters.entity == GNUTLS_SERVER ||
+      sid->size > TLS_MAX_SESSION_ID_SIZE)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  memset (&session->internals.resumed_security_parameters, 0,
+          sizeof (session->internals.resumed_security_parameters));
+
+  session->internals.resumed_security_parameters.session_id_size = sid->size;
+  memcpy(session->internals.resumed_security_parameters.session_id,
+  	sid->data, sid->size);
+
+  return 0;
+}
